@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevlp8k.c,v 1.2 2010/07/10 22:02:22 Arabidopsis Exp $*/
+/* $Id$*/
 
 /* EPSON LP-8000 ESC-sequence Laser Printer driver for Ghostscript.
 
@@ -30,29 +30,27 @@ encourage me to do it. (The only available in our laboratory "Operation
 guide" in Japanese does not contain any information about it. And LP-8000
 driver for Japanese Windows does not support this mode either.)
 
-
-The output data format is the following. 
+The output data format is the following.
 
 1. Initialization string, pretty long and sophisticated, I don't know why it
-was necessary.  
+was necessary.
 
 2. Data bits for each line. The most general format includes both starting X
 and Y values as well as data type (simple or compressed).
 
 3. Termination string.
 
-
-	DATA FORMATS
+        DATA FORMATS
 
 1. A simple (non-compressed) data format. By evident reasons it is NOT
 SUPPORTED by the driver and is discussed here just as a starting point for
 the future explanations. "\035" here is an alias for 0x1d ESC-character :
 
-"\035" "Starting X point in ASCII format" "X" 
+"\035" "Starting X point in ASCII format" "X"
 "\035" "Starting Y point in ASCII format" "Y"
-"\035" "Number of data BYTES for this printer line in ASCII format" ";" 
-"Number of POINTS to print in this line (equals to the 
-(Number of BYTES)*8)" ";" 
+"\035" "Number of data BYTES for this printer line in ASCII format" ";"
+"Number of POINTS to print in this line (equals to the
+(Number of BYTES)*8)" ";"
 "1;obi{I" "data BYTES for this line in BINARY format"
 
 Both X and Y printer coordinates are 60 pixels shifted from the corresponding
@@ -61,17 +59,17 @@ example, 1 inch left margin requires the value of 300 - 60 = 240 for
 starting X printer coordinate. Similar, 1.5 inch top margin requires Y
 values to start from 300*1.5 - 60 = 390.
 
-The shortest possible abbreviation for the simple data format string is 
+The shortest possible abbreviation for the simple data format string is
 
-"\035" "Starting Y point in ASCII format" "Y" 
+"\035" "Starting Y point in ASCII format" "Y"
 "\035" "Number of data BYTES for this printer line in ASCII format" ";"
-"Number of POINTS to print in this line (equals to the 
-(Number of BYTES)*8)" ";" 
+"Number of POINTS to print in this line (equals to the
+(Number of BYTES)*8)" ";"
 "1;obi{I" "data BYTES for this line in BINARY format"
 
 In this case the value of the starting X point is assumed to be equal to
 that for the previous line.
- 
+
 An example of the data output for 2 printer lines
 
 "\035"315X"\035"240Y"\035"2;16;1;obi{I"0ff0""\035"241Y"\035"3;24;1;obi{I"0f000f"
@@ -87,10 +85,10 @@ finally 4 black points.
 
 General description is as follows.
 
-"\035" "Starting X point in ASCII format" "X" 
+"\035" "Starting X point in ASCII format" "X"
 "\035" "Starting Y point in ASCII format" "Y"
-"\035" "3bcI" 
-"\035" "Total number of compressed BYTES in ASCII format" ";" 
+"\035" "3bcI"
+"\035" "Total number of compressed BYTES in ASCII format" ";"
 "Number of POINTS to print in this line" ";"
 "1;obi{I" "compressed data BYTES for this line in BINARY format"
 "\035" "0bcI"
@@ -100,11 +98,11 @@ of the compressed data format, respectively.  As in the discussed above case
 of a non-compressed data format, the shortest abbreviation has the form of
 
 "\035" "Starting Y point in ASCII format" "Y"
-"\035" "Total number of compressed BYTES in ASCII format" ";" 
+"\035" "Total number of compressed BYTES in ASCII format" ";"
 "Number of POINTS to print in this line" ";"
 "1;obi{I" "compressed data BYTES for this line in BINARY format"
 
-COMPRESSED DATA BYTES FORMAT has the form of 
+COMPRESSED DATA BYTES FORMAT has the form of
 
 "d1 d2 d3 d4 d4 count_d4 d5 d6 d6 count_d6 ... d(n-1) d(n-1) count_d(n-1) dn"
 
@@ -118,7 +116,7 @@ bytes somewhere in the data stream, they MUST follow by zero.
 Example of 2 compressed data strings.
 
 "\035"105X"\035"320Y"\035"3bcI"\035"3;2048;1;obi{I"0000fe"
-"\035"105X"\035"321Y"\035"11;2048;1;obi{I"0000021fffffe5fc000011" 
+"\035"105X"\035"321Y"\035"11;2048;1;obi{I"0000021fffffe5fc000011"
 
 The first one containing 3 bytes of compressed data will result in empty
 (zero) line of 2048 blank points started from X=105, Y=320. The second one
@@ -131,7 +129,6 @@ the byte boundary. I did it for the sake of simplicity. One more argument in
 favor of this step is that the error of positioning does not exceed (7 /
 300) inches or (7 / 118) cm, that is 0.6 mm, which is negligible, I guess.
 
-
 ADDITIONAL INFORMATION
 
 It is also possible to use LP-8000 printer with 180x180 DPI resolution as an
@@ -140,7 +137,7 @@ the value 0x30 in static const char ibmpro_init_string[]. Decimal 36
 fixes the 1,5 times elongation along the vertical axis. It is also
 recommended to choose the appropriate values for all margins. In my case it
 was 0.2, 0.6, 0, 0.3 in the device descriptor instead of the 0.2, 0.95, 0,
-1.0 
+1.0
 
 Nevertheless, typical Latex file looked so ugly after printing in this mode,
 that I preferred to spend several days for hacking the format of the Japanese
@@ -148,21 +145,20 @@ Windows printer output for 300 DPI resolution and create my own driver.
 
 Any suggestions, corrections, critical comments, etc. are welcome!
 
-Oleg Fat'yanov  <faty1@rlem.titech.ac.jp> 
+Oleg Fat'yanov  <faty1@rlem.titech.ac.jp>
 
 */
 
-
 #include "gdevprn.h"
 
 #ifndef X_DPI
 #define X_DPI 300
 #endif
-     
+
 #ifndef Y_DPI
 #define Y_DPI 300
 #endif
-     
+
 #define L_MARGIN 0.25
 #define B_MARGIN 0.25
 #define R_MARGIN 0.25
@@ -172,175 +168,169 @@ static dev_proc_print_page(lp8000_print_page);
 
 gx_device_printer far_data gs_lp8000_device =
   prn_device(prn_std_procs, "lp8000",
-	DEFAULT_WIDTH_10THS,
-	DEFAULT_HEIGHT_10THS,
-	X_DPI, Y_DPI,
-	L_MARGIN, B_MARGIN, R_MARGIN, T_MARGIN,
-	1, lp8000_print_page);
-                                          
-       
+        DEFAULT_WIDTH_10THS,
+        DEFAULT_HEIGHT_10THS,
+        X_DPI, Y_DPI,
+        L_MARGIN, B_MARGIN, R_MARGIN, T_MARGIN,
+        1, lp8000_print_page);
+
 static int
 lp8000_print_page(gx_device_printer *pdev, FILE *prn_stream)
 {
 
         int line_size = gdev_mem_bytes_per_scan_line((gx_device *)pdev);
         int in_size = line_size;
-                              
-	byte *buf1 = (byte *)gs_malloc(pdev->memory, in_size, 1, "lp8000_print_page(buf1)");
+
+        byte *buf1 = (byte *)gs_malloc(pdev->memory, in_size, 1, "lp8000_print_page(buf1)");
         byte *buf2 = (byte *)gs_malloc(pdev->memory, in_size, 1, "lp8000_print_page(buf2)");
         byte *in = buf1;
         byte *out = buf2;
-                        
-        int lnum, top, bottom, left, width;
-        int count, i, left1, left2, left0;
 
-/* Check  memory allocations  */         
-        
+        int lnum, top, bottom, left, width;
+        int count, left1, left2, left0;
+
+/* Check  memory allocations  */
+
         if ( buf1 == 0 || buf2 == 0 )
         {       if ( buf1 )
-        	gs_free(pdev->memory, (char *)buf1, in_size, 1, "lp8000_print_page(buf1)");
-                
+                gs_free(pdev->memory, (char *)buf1, in_size, 1, "lp8000_print_page(buf1)");
+
                 if ( buf2 )
                 gs_free(pdev->memory, (char *)buf2, in_size, 1, "lp8000_print_page(buf2)");
-          
+
           return_error(gs_error_VMerror);
-      	}
+        }
 
 /* Initialize the printer */
-       
-	fwrite("\033\001@EJL \n",1,8,prn_stream);
-	fwrite("@EJL EN LA=ESC/PAGE\n",1,20,prn_stream);
-	fwrite("\035rhE\033\001@EJL \n",1,12,prn_stream);
-	fwrite("@EJL SE LA=ESC/PAGE\n",1,20,prn_stream);
-	fwrite("@EJL SET PU=1 PS=A4 ZO=OFF\n",1,27,prn_stream);
-	fwrite("@EJL EN LA=ESC/PAGE\n",1,20,prn_stream);
-	fwrite("\0350;0.24muE\0352;300;300drE",1,23,prn_stream);
-	fwrite("\0350;300;300drE\0351tsE\0351mmE",1,23,prn_stream);
-	fwrite("\0357isE\0355iaF\0355ipP\03514psE\0350poE",1,26,prn_stream);
-	fwrite("\03560;60loE\0350X\0350Y",1,15,prn_stream);
-	fwrite("\0350;0;2360;3388caE",1,17,prn_stream);
-	fwrite("\0351cmE\0350alfP",1,11,prn_stream);
-	fwrite("\0350affP\0350boP\0350abP",1,16,prn_stream);
-	fwrite("\0354ilG\0350bcI\0350sarG",1,16,prn_stream);
-	fwrite("\0351;0;100spE\0352owE",1,16,prn_stream);
-	
-/* Here the common part of the initialization string ends */
 
+        fwrite("\033\001@EJL \n",1,8,prn_stream);
+        fwrite("@EJL EN LA=ESC/PAGE\n",1,20,prn_stream);
+        fwrite("\035rhE\033\001@EJL \n",1,12,prn_stream);
+        fwrite("@EJL SE LA=ESC/PAGE\n",1,20,prn_stream);
+        fwrite("@EJL SET PU=1 PS=A4 ZO=OFF\n",1,27,prn_stream);
+        fwrite("@EJL EN LA=ESC/PAGE\n",1,20,prn_stream);
+        fwrite("\0350;0.24muE\0352;300;300drE",1,23,prn_stream);
+        fwrite("\0350;300;300drE\0351tsE\0351mmE",1,23,prn_stream);
+        fwrite("\0357isE\0355iaF\0355ipP\03514psE\0350poE",1,26,prn_stream);
+        fwrite("\03560;60loE\0350X\0350Y",1,15,prn_stream);
+        fwrite("\0350;0;2360;3388caE",1,17,prn_stream);
+        fwrite("\0351cmE\0350alfP",1,11,prn_stream);
+        fwrite("\0350affP\0350boP\0350abP",1,16,prn_stream);
+        fwrite("\0354ilG\0350bcI\0350sarG",1,16,prn_stream);
+        fwrite("\0351;0;100spE\0352owE",1,16,prn_stream);
+
+/* Here the common part of the initialization string ends */
 
 /* Calculate the PRINTER_LEFT_MARGIN = device_left_margin - 60 adjusted to
 the byte boundary. Save this value for future comparison and set the
 starting X value of the printer line.
 */
- 	left1  =  (int) (L_MARGIN * pdev->x_pixels_per_inch) - 60;
- 	left1 = (left1 >> 3) << 3;
-	left0 = left1;
+        left1  =  (int) (L_MARGIN * pdev->x_pixels_per_inch) - 60;
+        left1 = (left1 >> 3) << 3;
+        left0 = left1;
 
-	fwrite("\035",1,1,prn_stream);
+        fwrite("\035",1,1,prn_stream);
         fprintf(prn_stream,"%d",left1);
         fwrite("X",1,1,prn_stream);
-        
-	/* Set the compressed data format */
+
+        /* Set the compressed data format */
         fwrite("\0353bcI",1,5,prn_stream);
-      
-      	top = T_MARGIN * pdev->y_pixels_per_inch;
+
+        top = T_MARGIN * pdev->y_pixels_per_inch;
         bottom = pdev->height - B_MARGIN * pdev->y_pixels_per_inch;
-        
-	left  = ( (int) (L_MARGIN * pdev->x_pixels_per_inch) ) >> 3 ;
- 	width = ((pdev->width - (int)(R_MARGIN * pdev->x_pixels_per_inch)) >> 3) - left;
- 
-	/*
-	** Print the page:
-	*/
 
-	for ( lnum = top; lnum < bottom ; )
+        left  = ( (int) (L_MARGIN * pdev->x_pixels_per_inch) ) >> 3 ;
+        width = ((pdev->width - (int)(R_MARGIN * pdev->x_pixels_per_inch)) >> 3) - left;
 
+        /*
+        ** Print the page:
+        */
 
-	{	
-		byte *in_data;
-		byte *inp;
-		byte *in_end;
-		byte *outp;
-		register byte *p, *q;
-		int lcnt;
+        for ( lnum = top; lnum < bottom ; )
 
-		/*
-		** Check buffer for 0 data.
-		*/
+        {
+                byte *in_data;
+                byte *inp;
+                byte *in_end;
+                byte *outp;
+                register byte *p, *q;
+                int lcnt;
 
-		gdev_prn_get_bits(pdev, lnum, in, &in_data);
-		while ( in_data[0] == 0 &&
-		        !memcmp((char *)in_data, (char *)in_data + 1, line_size - 1) &&
-		        lnum < bottom )
-	    	{	
-			lnum++;
-			gdev_prn_get_bits(pdev, lnum, in, &in_data);
-		}
+                /*
+                ** Check buffer for 0 data.
+                */
 
-		if(lnum == bottom ) break;	
-		/* finished with this page */
+                gdev_prn_get_bits(pdev, lnum, in, &in_data);
+                while ( in_data[0] == 0 &&
+                        !memcmp((char *)in_data, (char *)in_data + 1, line_size - 1) &&
+                        lnum < bottom )
+                {
+                        lnum++;
+                        gdev_prn_get_bits(pdev, lnum, in, &in_data);
+                }
 
+                if(lnum == bottom ) break;
+                /* finished with this page */
 
-		lcnt = gdev_prn_copy_scan_lines(pdev, lnum, in, in_size);
+                lcnt = gdev_prn_copy_scan_lines(pdev, lnum, in, in_size);
 
-		inp = in  + left;
-		in_end = inp + width;
+                inp = in  + left;
+                in_end = inp + width;
 
 /* Remove trailing 0s form the scan line data */
 
-		while (in_end > inp &&  in_end[-1] == 0) 
-		{
-		in_end--;
-		}
-		
+                while (in_end > inp &&  in_end[-1] == 0)
+                {
+                in_end--;
+                }
+
 /* Remove leading 0s form the scan line data */
-	
-	for(left2 = 0; inp < in_end && inp[0] == 0; inp++,left2++);
+
+        for(left2 = 0; inp < in_end && inp[0] == 0; inp++,left2++);
 
 /* Recalculate starting X value */
- 
-	left2 = left1 + (left2 << 3);
-
 
+        left2 = left1 + (left2 << 3);
 
 /* Compress  non-zero data for this line*/
 
-		outp = out;
+                outp = out;
 
  for( p = inp, q = inp + 1 ; q < in_end ; )
- 	{
-	if( *p != *q++ ) 
-	  { 
-	  /* 
-	  Copy non-repeated bytes 
-	  to the output buffer  
-	  */
-	  *outp++ = *p++;
-	  } 
-	  else 
-	    {
-	    for (count = 2; ( *p == *q ) && (q < in_end); q++, count++);
+        {
+        if( *p != *q++ )
+          {
+          /*
+          Copy non-repeated bytes
+          to the output buffer
+          */
+          *outp++ = *p++;
+          }
+          else
+            {
+            for (count = 2; ( *p == *q ) && (q < in_end); q++, count++);
 
-		/* 
-		Copy repeated bytes and counts to the output buffer. 
-		As long as count is <= 255, additional step is necessary 
-		for a long repeated sequence
-		*/
-	    	
-	    	while (count > 257)
-	    	{
-	    	*outp++ = *p;
-	    	*outp++ = *p;
-	    	*outp++ = 255;
-	    	p += 257;
-	    	count -=257;
-		}
-		*outp++ = *p;
-	    	*outp++ = *p;
-	    	*outp++ = count - 2;
-	    	p += count;
-		q = p+1;
-	    }		
-	} 
+                /*
+                Copy repeated bytes and counts to the output buffer.
+                As long as count is <= 255, additional step is necessary
+                for a long repeated sequence
+                */
+
+                while (count > 257)
+                {
+                *outp++ = *p;
+                *outp++ = *p;
+                *outp++ = 255;
+                p += 257;
+                count -=257;
+                }
+                *outp++ = *p;
+                *outp++ = *p;
+                *outp++ = count - 2;
+                p += count;
+                q = p+1;
+            }
+        }
 
 /* The next line is necessary just in case of a single non-repeated byte at
 the end of the input buffer */
@@ -348,59 +338,58 @@ the end of the input buffer */
 if (p == (in_end - 1)) *outp++ = *p;
 
 /* End of the compression procedure */
-	
 
 /* Set a new value of the starting X point, if necessary  */
 
 if (left2 != left0)
-	{
-	left0 = left2;
-	fwrite("\035",1,1,prn_stream);
+        {
+        left0 = left2;
+        fwrite("\035",1,1,prn_stream);
         fprintf(prn_stream,"%d",left2);
         fwrite("X",1,1,prn_stream);
         }
 
-/* Output the data string to the printer. 
+/* Output the data string to the printer.
 Y coordinate of the printer equals (lnum - 60)
-*/        	
+*/
 
         fwrite("\035",1,1,prn_stream);
         fprintf(prn_stream,"%d",lnum-60);
         fwrite("Y\035",1,2,prn_stream);
         fprintf(prn_stream,"%d;",(outp - out));
-	fprintf(prn_stream,"%d;",(in_end - inp) << 3); 
-	fwrite("1;0bi{I",1,7,prn_stream);       
-        fwrite(out,1,(outp - out),prn_stream);                 
-        
+        fprintf(prn_stream,"%d;",(in_end - inp) << 3);
+        fwrite("1;0bi{I",1,7,prn_stream);
+        fwrite(out,1,(outp - out),prn_stream);
+
         lnum++;
-        
+
         }
 
 /* Send the termination string */
 
-        fwrite("\0350bcI",1,5,prn_stream);                        
-	fwrite("\0351coO",1,5,prn_stream);
-    	fwrite("\035rhE",1,4,prn_stream); 
-        
+        fwrite("\0350bcI",1,5,prn_stream);
+        fwrite("\0351coO",1,5,prn_stream);
+        fwrite("\035rhE",1,4,prn_stream);
+
         fwrite("\033\001@EJL \n",1,8,prn_stream);
-	fwrite("@EJL SE LA=ESC/PAGE\n",1,20,prn_stream);
-	fwrite("@EJL SET PU=1 PS=A4 ZO=OFF\n",1,27,prn_stream);
-	fwrite("@EJL EN LA=ESC/PAGE\n",1,20,prn_stream);
-	fwrite("\0350;0.24muE\0352;300;300drE",1,23,prn_stream);
-	fwrite("\0350;300;300drE\0351tsE\0351mmE",1,23,prn_stream);
-	fwrite("\0357isE\0355iaF\0355ipP\03514psE\0350poE",1,26,prn_stream);
-	fwrite("\03560;60loE\0350X\0350Y",1,15,prn_stream);
-	fwrite("\0350;0;2360;3388caE",1,17,prn_stream);
-	fwrite("\0351cmE\0350alfP",1,11,prn_stream);
-	fwrite("\0350affP\0350boP\0350abP",1,16,prn_stream);
-	fwrite("\0354ilG\0350bcI\0350sarG",1,16,prn_stream);
-	fwrite("\035rhE",1,4,prn_stream);
-	fwrite("\033\001@EJL \n",1,8,prn_stream);
-	fwrite("\033\001@EJL \n",1,8,prn_stream);
-	
-	fflush(prn_stream);
-	
-	gs_free(pdev->memory, (char *)buf2, in_size, 1, "lp8000_print_page(buf2)");
-	gs_free(pdev->memory, (char *)buf1, in_size, 1, "lp8000_print_page(buf1)");
-	return 0;
+        fwrite("@EJL SE LA=ESC/PAGE\n",1,20,prn_stream);
+        fwrite("@EJL SET PU=1 PS=A4 ZO=OFF\n",1,27,prn_stream);
+        fwrite("@EJL EN LA=ESC/PAGE\n",1,20,prn_stream);
+        fwrite("\0350;0.24muE\0352;300;300drE",1,23,prn_stream);
+        fwrite("\0350;300;300drE\0351tsE\0351mmE",1,23,prn_stream);
+        fwrite("\0357isE\0355iaF\0355ipP\03514psE\0350poE",1,26,prn_stream);
+        fwrite("\03560;60loE\0350X\0350Y",1,15,prn_stream);
+        fwrite("\0350;0;2360;3388caE",1,17,prn_stream);
+        fwrite("\0351cmE\0350alfP",1,11,prn_stream);
+        fwrite("\0350affP\0350boP\0350abP",1,16,prn_stream);
+        fwrite("\0354ilG\0350bcI\0350sarG",1,16,prn_stream);
+        fwrite("\035rhE",1,4,prn_stream);
+        fwrite("\033\001@EJL \n",1,8,prn_stream);
+        fwrite("\033\001@EJL \n",1,8,prn_stream);
+
+        fflush(prn_stream);
+
+        gs_free(pdev->memory, (char *)buf2, in_size, 1, "lp8000_print_page(buf2)");
+        gs_free(pdev->memory, (char *)buf1, in_size, 1, "lp8000_print_page(buf1)");
+        return 0;
 }
