@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
-  This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdts.c,v 1.10 2008/05/04 14:34:47 Arabidopsis Exp $ */
+/* $Id$ */
 /* Text state management for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
@@ -29,6 +23,7 @@
 #include "gdevpdtf.h"		/* for pdfont->FontType */
 #include "gdevpdts.h"
 #include "gdevpdtt.h"
+#include "gdevpdti.h"
 
 /* ================ Types and structures ================ */
 
@@ -105,8 +100,8 @@ static const pdf_text_state_t ts_default = {
 };
 /* GC descriptor */
 gs_private_st_ptrs2(st_pdf_text_state, pdf_text_state_t,  "pdf_text_state_t",
-		    pdf_text_state_enum_ptrs, pdf_text_state_reloc_ptrs,
-		    in.pdfont, out.pdfont);
+                    pdf_text_state_enum_ptrs, pdf_text_state_reloc_ptrs,
+                    in.pdfont, out.pdfont);
 
 /* ================ Procedures ================ */
 
@@ -126,24 +121,24 @@ append_text_move(pdf_text_state_t *pts, floatp dw)
     double rounded;
 
     if (count > 0 && pts->buffer.moves[count - 1].index == pos) {
-	/* Merge adjacent moves. */
-	dw += pts->buffer.moves[--count].amount;
+        /* Merge adjacent moves. */
+        dw += pts->buffer.moves[--count].amount;
     }
     /* Round dw if it's very close to an integer. */
     rounded = floor(dw + 0.5);
     if (fabs(dw - rounded) < 0.001)
-	dw = rounded;
+        dw = rounded;
     if (dw < -MAX_USER_COORD) {
-	/* Acrobat reader 4.0c, 5.0 can't handle big offsets. 
-	   Adobe Reader 6 can. */
-	return -1;
+        /* Acrobat reader 4.0c, 5.0 can't handle big offsets.
+           Adobe Reader 6 can. */
+        return -1;
     }
     if (dw != 0) {
-	if (count == MAX_TEXT_BUFFER_MOVES)
-	    return -1;
-	pts->buffer.moves[count].index = pos;
-	pts->buffer.moves[count].amount = dw;
-	++count;
+        if (count == MAX_TEXT_BUFFER_MOVES)
+            return -1;
+        pts->buffer.moves[count].index = pos;
+        pts->buffer.moves[count].amount = dw;
+        ++count;
     }
     pts->buffer.count_moves = count;
     return 0;
@@ -159,20 +154,20 @@ set_text_distance(gs_point *pdist, floatp dx, floatp dy, const gs_matrix *pmat)
     double rounded;
 
     if (code == gs_error_undefinedresult) {
-	/* The CTM is degenerate.
-	   Can't know the distance in user space.
-	   Set zero because we believe it is not important for rendering.
-	   We want to copy the text to PDF to make it searchable.
-	   Bug 689006.
-	 */
-	pdist->x = pdist->y = 0;
+        /* The CTM is degenerate.
+           Can't know the distance in user space.
+           Set zero because we believe it is not important for rendering.
+           We want to copy the text to PDF to make it searchable.
+           Bug 689006.
+         */
+        pdist->x = pdist->y = 0;
     } else if (code < 0)
-	return code;
+        return code;
     /* If the distance is very close to integers, round it. */
     if (fabs(pdist->x - (rounded = floor(pdist->x + 0.5))) < 0.0005)
-	pdist->x = rounded;
+        pdist->x = rounded;
     if (fabs(pdist->y - (rounded = floor(pdist->y + 0.5))) < 0.0005)
-	pdist->y = rounded;
+        pdist->y = rounded;
     return 0;
 }
 
@@ -183,7 +178,7 @@ static bool
 matrix_is_compatible(const gs_matrix *pmat1, const gs_matrix *pmat2)
 {
     return (pmat2->xx == pmat1->xx && pmat2->xy == pmat1->xy &&
-	    pmat2->yx == pmat1->yx && pmat2->yy == pmat1->yy);
+            pmat2->yx == pmat1->yx && pmat2->yy == pmat1->yy);
 }
 
 /*
@@ -196,51 +191,59 @@ add_text_delta_move(gx_device_pdf *pdev, const gs_matrix *pmat)
     pdf_text_state_t *const pts = pdev->text->text_state;
 
     if (matrix_is_compatible(pmat, &pts->in.matrix)) {
-	double dx = pmat->tx - pts->in.matrix.tx,
-	    dy = pmat->ty - pts->in.matrix.ty;
-	gs_point dist;
-	double dw, dnotw, tdw;
-	int code;
+        double dx = pmat->tx - pts->in.matrix.tx,
+            dy = pmat->ty - pts->in.matrix.ty;
+        gs_point dist;
+        double dw, dnotw, tdw;
+        int code;
 
-	code = set_text_distance(&dist, dx, dy, pmat);
-	if (code < 0)
-	    return code;
-	if (pts->wmode)
-	    dw = dist.y, dnotw = dist.x;
-	else
-	    dw = dist.x, dnotw = dist.y;
-	if (dnotw == 0 && pts->buffer.count_chars > 0 &&
-	    /*
-	     * Acrobat Reader limits the magnitude of user-space
-	     * coordinates.  Also, AR apparently doesn't handle large
-	     * positive movement values (negative X displacements), even
-	     * though the PDF Reference says this bug was fixed in AR3.
-	     *
-	     * Old revisions used the upper threshold 1000 for tdw,
-	     * but it appears too big when a font sets a too big
-	     * character width in setcachedevice. Particularly this happens 
-	     * with a Type 3 font generated by Aldus Freehand 4.0
-	     * to represent a texture - see bug #687051.
-	     * The problem is that when the Widths is multiplied 
-	     * to the font size, the viewer represents the result
-	     * with insufficient fraction bits to represent the precise width.
-	     * We work around that problem here restricting tdw
-	     * with a smaller threshold 990. Our intention is to 
-	     * disable Tj when the real glyph width appears smaller
-	     * than 1% of the width specified in setcachedevice.
-	     * A Td instruction will be generated instead.
-	     * Note that the value 990 is arbitrary and may need a
-	     * further adjustment.
-	     */
-	    (tdw = dw * -1000.0 / pts->in.size,
-	     tdw >= -MAX_USER_COORD && tdw < 990)
-	    ) {
-	    /* Use TJ. */
-	    int code = append_text_move(pts, tdw);
+        code = set_text_distance(&dist, dx, dy, pmat);
+        if (code < 0)
+            return code;
+        if (pts->wmode)
+            dw = dist.y, dnotw = dist.x;
+        else
+            dw = dist.x, dnotw = dist.y;
+        if (dnotw == 0 && pts->buffer.count_chars > 0 &&
+            /*
+             * Acrobat Reader limits the magnitude of user-space
+             * coordinates.  Also, AR apparently doesn't handle large
+             * positive movement values (negative X displacements), even
+             * though the PDF Reference says this bug was fixed in AR3.
+             *
+             * Old revisions used the upper threshold 1000 for tdw,
+             * but it appears too big when a font sets a too big
+             * character width in setcachedevice. Particularly this happens
+             * with a Type 3 font generated by Aldus Freehand 4.0
+             * to represent a texture - see bug #687051.
+             * The problem is that when the Widths is multiplied
+             * to the font size, the viewer represents the result
+             * with insufficient fraction bits to represent the precise width.
+             * We work around that problem here restricting tdw
+             * with a smaller threshold 990. Our intention is to
+             * disable Tj when the real glyph width appears smaller
+             * than 1% of the width specified in setcachedevice.
+             * A Td instruction will be generated instead.
+             * Note that the value 990 is arbitrary and may need a
+             * further adjustment.
+             */
+             /* Revised the above. It seems unreasonable to use a fixed
+              * value which is not based on the point size, when the problem is
+              * caused by a large point size being multiplied by the width. The
+              * original fix also caused bitmap fonts (from PCL and other sources)
+              * to fail to use kerning, as these fonts are scaled to 1 point and
+              * therefore use large kerning values. Instead we check the kerned value
+              * multiplied by the point size of the font.
+              */
+            (tdw = dw * -1000.0 / pts->in.size,
+             tdw >= -MAX_USER_COORD && (tdw * pts->in.size) < MAX_USER_COORD)
+            ) {
+            /* Use TJ. */
+            int code = append_text_move(pts, tdw);
 
-	    if (code >= 0)
-		goto finish;
-	}
+            if (code >= 0)
+                goto finish;
+        }
     }
     return -1;
  finish:
@@ -263,38 +266,38 @@ pdf_set_text_matrix(gx_device_pdf * pdev)
 
     pts->use_leading = false;
     if (matrix_is_compatible(&pts->out.matrix, &pts->in.matrix)) {
-	gs_point dist;
-	int code;
+        gs_point dist;
+        int code;
 
-	code = set_text_distance(&dist, pts->start.x - pts->line_start.x,
-			  pts->start.y - pts->line_start.y, &pts->in.matrix);
-	if (code < 0)
-	    return code;
-	if (dist.x == 0 && dist.y < 0) {
-	    /* Use TL, if needed, and T* or '. */
-	    float dist_y = (float)-dist.y;
+        code = set_text_distance(&dist, pts->start.x - pts->line_start.x,
+                          pts->start.y - pts->line_start.y, &pts->in.matrix);
+        if (code < 0)
+            return code;
+        if (dist.x == 0 && dist.y < 0) {
+            /* Use TL, if needed, and T* or '. */
+            float dist_y = (float)-dist.y;
 
-	    if (fabs(pts->leading - dist_y) > 0.0005) {
-		pprintg1(s, "%g TL\n", dist_y);
-		pts->leading = dist_y;
-	    }
-	    pts->use_leading = true;
-	} else {
-	    /* Use Td. */
-	    pprintg2(s, "%g %g Td\n", dist.x, dist.y);
-	}
+            if (fabs(pts->leading - dist_y) > 0.0005) {
+                pprintg1(s, "%g TL\n", dist_y);
+                pts->leading = dist_y;
+            }
+            pts->use_leading = true;
+        } else {
+            /* Use Td. */
+            pprintg2(s, "%g %g Td\n", dist.x, dist.y);
+        }
     } else {			/* Use Tm. */
-	/*
-	 * See stream_to_text in gdevpdfu.c for why we need the following
-	 * matrix adjustments.
-	 */
-	double sx = 72.0 / pdev->HWResolution[0],
-	    sy = 72.0 / pdev->HWResolution[1];
+        /*
+         * See stream_to_text in gdevpdfu.c for why we need the following
+         * matrix adjustments.
+         */
+        double sx = 72.0 / pdev->HWResolution[0],
+            sy = 72.0 / pdev->HWResolution[1];
 
-	pprintg6(s, "%g %g %g %g %g %g Tm\n",
-		 pts->in.matrix.xx * sx, pts->in.matrix.xy * sy,
-		 pts->in.matrix.yx * sx, pts->in.matrix.yy * sy,
-		 pts->start.x * sx, pts->start.y * sy);
+        pprintg6(s, "%g %g %g %g %g %g Tm\n",
+                 pts->in.matrix.xx * sx, pts->in.matrix.xy * sy,
+                 pts->in.matrix.yx * sx, pts->in.matrix.yy * sy,
+                 pts->start.x * sx, pts->start.y * sy);
     }
     pts->line_start.x = pts->start.x;
     pts->line_start.y = pts->start.y;
@@ -311,11 +314,11 @@ pdf_text_state_t *
 pdf_text_state_alloc(gs_memory_t *mem)
 {
     pdf_text_state_t *pts =
-	gs_alloc_struct(mem, pdf_text_state_t, &st_pdf_text_state,
-			"pdf_text_state_alloc");
+        gs_alloc_struct(mem, pdf_text_state_t, &st_pdf_text_state,
+                        "pdf_text_state_alloc");
 
     if (pts == 0)
-	return 0;
+        return 0;
     *pts = ts_default;
     return pts;
 }
@@ -391,35 +394,35 @@ flush_text_buffer(gx_device_pdf *pdev)
     stream *s = pdev->strm;
 
     if (pts->buffer.count_chars != 0) {
-	pdf_font_resource_t *pdfont = pts->in.pdfont;
-	int code = pdf_assign_font_object_id(pdev, pdfont);
+        pdf_font_resource_t *pdfont = pts->in.pdfont;
+        int code = pdf_assign_font_object_id(pdev, pdfont);
 
-	if (code < 0)
-	    return code;
-	code = pdf_add_resource(pdev, pdev->substream_Resources, "/Font", (pdf_resource_t *)pdfont);
-	if (code < 0)
-	    return code;
+        if (code < 0)
+            return code;
+        code = pdf_add_resource(pdev, pdev->substream_Resources, "/Font", (pdf_resource_t *)pdfont);
+        if (code < 0)
+            return code;
     }
     if (pts->buffer.count_moves > 0) {
-	int i, cur = 0;
+        int i, cur = 0;
 
-	if (pts->use_leading)
-	    stream_puts(s, "T*");
-	stream_puts(s, "[");
-	for (i = 0; i < pts->buffer.count_moves; ++i) {
-	    int next = pts->buffer.moves[i].index;
+        if (pts->use_leading)
+            stream_puts(s, "T*");
+        stream_puts(s, "[");
+        for (i = 0; i < pts->buffer.count_moves; ++i) {
+            int next = pts->buffer.moves[i].index;
 
-	    pdf_put_string(pdev, pts->buffer.chars + cur, next - cur);
-	    pprintg1(s, "%g", pts->buffer.moves[i].amount);
-	    cur = next;
-	}
-	if (pts->buffer.count_chars > cur)
-	    pdf_put_string(pdev, pts->buffer.chars + cur,
-			   pts->buffer.count_chars - cur);
-	stream_puts(s, "]TJ\n");
+            pdf_put_string(pdev, pts->buffer.chars + cur, next - cur);
+            pprintg1(s, "%g", pts->buffer.moves[i].amount);
+            cur = next;
+        }
+        if (pts->buffer.count_chars > cur)
+            pdf_put_string(pdev, pts->buffer.chars + cur,
+                           pts->buffer.count_chars - cur);
+        stream_puts(s, "]TJ\n");
     } else {
-	pdf_put_string(pdev, pts->buffer.chars, pts->buffer.count_chars);
-	stream_puts(s, (pts->use_leading ? "'\n" : "Tj\n"));
+        pdf_put_string(pdev, pts->buffer.chars, pts->buffer.count_chars);
+        stream_puts(s, (pts->use_leading ? "'\n" : "Tj\n"));
     }
     pts->buffer.count_chars = 0;
     pts->buffer.count_moves = 0;
@@ -438,59 +441,59 @@ sync_text_state(gx_device_pdf *pdev)
     int code;
 
     if (pts->buffer.count_chars == 0)
-	return 0;		/* nothing to output */
+        return 0;		/* nothing to output */
 
-    if (pts->continue_line) 
-	return flush_text_buffer(pdev);
+    if (pts->continue_line)
+        return flush_text_buffer(pdev);
 
     /* Bring text state parameters up to date. */
 
     if (pts->out.character_spacing != pts->in.character_spacing) {
-	pprintg1(s, "%g Tc\n", pts->in.character_spacing);
-	pts->out.character_spacing = pts->in.character_spacing;
+        pprintg1(s, "%g Tc\n", pts->in.character_spacing);
+        pts->out.character_spacing = pts->in.character_spacing;
     }
 
     if (pts->out.pdfont != pts->in.pdfont || pts->out.size != pts->in.size) {
-	pdf_font_resource_t *pdfont = pts->in.pdfont;
+        pdf_font_resource_t *pdfont = pts->in.pdfont;
 
-	code = pdf_assign_font_object_id(pdev, pdfont);
-	if (code < 0)
-	    return code;
-	pprints1(s, "/%s ", pdfont->rname);
-	pprintg1(s, "%g Tf\n", pts->in.size);
-	pts->out.pdfont = pdfont;
-	pts->out.size = pts->in.size;
-	/*
-	 * In PDF, the only place to specify WMode is in the CMap
-	 * (a.k.a. Encoding) of a Type 0 font.
-	 */
-	pts->wmode =
-	    (pdfont->FontType == ft_composite ?
-	     pdfont->u.type0.WMode : 0);
-	code = pdf_used_charproc_resources(pdev, pdfont);
-	if (code < 0)
-	    return code;
+        code = pdf_assign_font_object_id(pdev, pdfont);
+        if (code < 0)
+            return code;
+        pprints1(s, "/%s ", pdfont->rname);
+        pprintg1(s, "%g Tf\n", pts->in.size);
+        pts->out.pdfont = pdfont;
+        pts->out.size = pts->in.size;
+        /*
+         * In PDF, the only place to specify WMode is in the CMap
+         * (a.k.a. Encoding) of a Type 0 font.
+         */
+        pts->wmode =
+            (pdfont->FontType == ft_composite ?
+             pdfont->u.type0.WMode : 0);
+        code = pdf_used_charproc_resources(pdev, pdfont);
+        if (code < 0)
+            return code;
     }
 
     if (memcmp(&pts->in.matrix, &pts->out.matrix, sizeof(pts->in.matrix)) ||
-	 ((pts->start.x != pts->out_pos.x || pts->start.y != pts->out_pos.y) &&
-	  (pts->buffer.count_chars != 0 || pts->buffer.count_moves != 0))) {
-	/* pdf_set_text_matrix sets out.matrix = in.matrix */
-	code = pdf_set_text_matrix(pdev);
-	if (code < 0)
-	    return code;
+         ((pts->start.x != pts->out_pos.x || pts->start.y != pts->out_pos.y) &&
+          (pts->buffer.count_chars != 0 || pts->buffer.count_moves != 0))) {
+        /* pdf_set_text_matrix sets out.matrix = in.matrix */
+        code = pdf_set_text_matrix(pdev);
+        if (code < 0)
+            return code;
     }
 
     if (pts->out.render_mode != pts->in.render_mode) {
-	pprintg1(s, "%g Tr\n", pts->in.render_mode);
-	pts->out.render_mode = pts->in.render_mode;
+        pprintg1(s, "%g Tr\n", pts->in.render_mode);
+        pts->out.render_mode = pts->in.render_mode;
     }
 
     if (pts->out.word_spacing != pts->in.word_spacing) {
-	if (memchr(pts->buffer.chars, 32, pts->buffer.count_chars)) {
-	    pprintg1(s, "%g Tw\n", pts->in.word_spacing);
-	    pts->out.word_spacing = pts->in.word_spacing;
-	}
+        if (memchr(pts->buffer.chars, 32, pts->buffer.count_chars)) {
+            pprintg1(s, "%g Tw\n", pts->in.word_spacing);
+            pts->out.word_spacing = pts->in.word_spacing;
+        }
     }
 
     return flush_text_buffer(pdev);
@@ -525,10 +528,10 @@ pdf_close_text_contents(gx_device_pdf *pdev)
  */
 bool
 pdf_render_mode_uses_stroke(const gx_device_pdf *pdev,
-			    const pdf_text_state_values_t *ptsv)
+                            const pdf_text_state_values_t *ptsv)
 {
     return ((ptsv->render_mode == 1 || ptsv->render_mode == 2 ||
-	    ptsv->render_mode == 5 || ptsv->render_mode == 6));
+            ptsv->render_mode == 5 || ptsv->render_mode == 6));
 }
 
 /*
@@ -551,35 +554,34 @@ pdf_set_text_wmode(gx_device_pdf *pdev, int wmode)
     pts->wmode = wmode;
 }
 
-
 /*
  * Set the stored client view of text state values.
  */
 int
 pdf_set_text_state_values(gx_device_pdf *pdev,
-			  const pdf_text_state_values_t *ptsv)
+                          const pdf_text_state_values_t *ptsv)
 {
     pdf_text_state_t *pts = pdev->text->text_state;
 
     if (pts->buffer.count_chars > 0) {
-	int code;
+        int code;
 
-	if (pts->in.character_spacing == ptsv->character_spacing &&
-	    pts->in.pdfont == ptsv->pdfont && pts->in.size == ptsv->size &&
-	    pts->in.render_mode == ptsv->render_mode &&
-	    pts->in.word_spacing == ptsv->word_spacing
-	    ) {
-	    if (!memcmp(&pts->in.matrix, &ptsv->matrix,
-			sizeof(pts->in.matrix)))
-		return 0;
-	    /* add_text_delta_move sets pts->in.matrix if successful */
-	    code = add_text_delta_move(pdev, &ptsv->matrix);
-	    if (code >= 0)
-		return 0;
-	}
-	code = sync_text_state(pdev);
-	if (code < 0)
-	    return code;
+        if (pts->in.character_spacing == ptsv->character_spacing &&
+            pts->in.pdfont == ptsv->pdfont && pts->in.size == ptsv->size &&
+            pts->in.render_mode == ptsv->render_mode &&
+            pts->in.word_spacing == ptsv->word_spacing
+            ) {
+            if (!memcmp(&pts->in.matrix, &ptsv->matrix,
+                sizeof(pts->in.matrix)))
+                return 0;
+            /* add_text_delta_move sets pts->in.matrix if successful */
+            code = add_text_delta_move(pdev, &ptsv->matrix);
+            if (code >= 0)
+                return 0;
+        }
+        code = sync_text_state(pdev);
+        if (code < 0)
+            return code;
     }
 
     pts->in = *ptsv;
@@ -593,7 +595,7 @@ pdf_set_text_state_values(gx_device_pdf *pdev,
  */
 int
 pdf_text_distance_transform(floatp wx, floatp wy, const pdf_text_state_t *pts,
-			    gs_point *ppt)
+                            gs_point *ppt)
 {
     return gs_distance_transform(wx, wy, &pts->in.matrix, ppt);
 }
@@ -617,45 +619,45 @@ pdf_text_position(const gx_device_pdf *pdev, gs_point *ppt)
  */
 int
 pdf_append_chars(gx_device_pdf * pdev, const byte * str, uint size,
-		 floatp wx, floatp wy, bool nobreak)
+                 floatp wx, floatp wy, bool nobreak)
 {
     pdf_text_state_t *pts = pdev->text->text_state;
     const byte *p = str;
     uint left = size;
 
     if (pts->buffer.count_chars == 0 && pts->buffer.count_moves == 0) {
-	pts->out_pos.x = pts->start.x = pts->in.matrix.tx;
-	pts->out_pos.y = pts->start.y = pts->in.matrix.ty;
+        pts->out_pos.x = pts->start.x = pts->in.matrix.tx;
+        pts->out_pos.y = pts->start.y = pts->in.matrix.ty;
     }
     while (left)
-	if (pts->buffer.count_chars == MAX_TEXT_BUFFER_CHARS ||
-	    (nobreak && pts->buffer.count_chars + left > MAX_TEXT_BUFFER_CHARS)) {
-	    int code = sync_text_state(pdev);
+        if (pts->buffer.count_chars == MAX_TEXT_BUFFER_CHARS ||
+            (nobreak && pts->buffer.count_chars + left > MAX_TEXT_BUFFER_CHARS)) {
+            int code = sync_text_state(pdev);
 
-	    if (code < 0)
-		return code;
-	    /* We'll keep a continuation of this line in the buffer,
-	     * but the current input parameters don't correspond to
-	     * the current position, because the text was broken in a
-	     * middle with unknown current point.
-	     * Don't change the output text state parameters
-	     * until input parameters are changed. 
-	     * pdf_set_text_state_values will reset the 'continue_line' flag 
-	     * at that time.
-	     */
-	    pts->continue_line = true;
-	} else {
-	    int code = pdf_open_page(pdev, PDF_IN_STRING);
-	    uint copy;
+            if (code < 0)
+                return code;
+            /* We'll keep a continuation of this line in the buffer,
+             * but the current input parameters don't correspond to
+             * the current position, because the text was broken in a
+             * middle with unknown current point.
+             * Don't change the output text state parameters
+             * until input parameters are changed.
+             * pdf_set_text_state_values will reset the 'continue_line' flag
+             * at that time.
+             */
+            pts->continue_line = true;
+        } else {
+            int code = pdf_open_page(pdev, PDF_IN_STRING);
+            uint copy;
 
-	    if (code < 0)
-		return code;
-	    copy = min(MAX_TEXT_BUFFER_CHARS - pts->buffer.count_chars, left);
-	    memcpy(pts->buffer.chars + pts->buffer.count_chars, p, copy);
-	    pts->buffer.count_chars += copy;
-	    p += copy;
-	    left -= copy;
-	}
+            if (code < 0)
+                return code;
+            copy = min(MAX_TEXT_BUFFER_CHARS - pts->buffer.count_chars, left);
+            memcpy(pts->buffer.chars + pts->buffer.count_chars, p, copy);
+            pts->buffer.count_chars += copy;
+            p += copy;
+            left -= copy;
+        }
     pts->in.matrix.tx += wx;
     pts->in.matrix.ty += wy;
     pts->out_pos.x += wx;
@@ -666,9 +668,9 @@ pdf_append_chars(gx_device_pdf * pdev, const byte * str, uint size,
 /* Check a new piece of charpath text to see if its safe to combine
  * with a previous text operation using text rendering modes.
  */
-bool pdf_compare_text_state_for_charpath(pdf_text_state_t *pts, gx_device_pdf *pdev, 
-					 gs_imager_state *pis, gs_font *font, 
-					 const gs_text_params_t *text)
+bool pdf_compare_text_state_for_charpath(pdf_text_state_t *pts, gx_device_pdf *pdev,
+                                         gs_imager_state *pis, gs_font *font,
+                                         const gs_text_params_t *text)
 {
     int code;
     float size;
@@ -677,32 +679,34 @@ bool pdf_compare_text_state_for_charpath(pdf_text_state_t *pts, gx_device_pdf *p
 
     /* check to ensure the new text has the same length as the saved text */
     if(text->size != pts->buffer.count_chars)
-	return(false);
+        return(false);
 
-    if(font->FontType == ft_user_defined)
-	return(false);
+    if(font->FontType == ft_user_defined ||
+        font->FontType == ft_PCL_user_defined ||
+        font->FontType == ft_GL2_stick_user_defined)
+        return(false);
 
     /* check to ensure the new text has the same data as the saved text */
     if(memcmp(text->data.bytes, &pts->buffer.chars, text->size))
-	return(false);
+        return(false);
 
     /* See if the same font is in use by checking the attahced pdfont resource for
      * the currrent font and comparing with the saved text state
      */
     code = pdf_attached_font_resource(pdev, font, &pdfont, NULL, NULL, NULL, NULL);
     if(code < 0)
-	return(false);
+        return(false);
 
     if(!pdfont || pdfont != pts->in.pdfont)
-	return(false);
+        return(false);
 
-    /* Check to see the new text starts at the same point as the saved text. 
+    /* Check to see the new text starts at the same point as the saved text.
      * NB! only check 2 decimal places, allow some slack in the match. This
      * still may prove to be too tight a requirement.
      */
-    if(fabs(pts->start.x - pis->current_point.x) > 0.01 || 
-       fabs(pts->start.y - pis->current_point.y) > 0.01) 
-	return(false);
+    if(fabs(pts->start.x - pis->current_point.x) > 0.01 ||
+       fabs(pts->start.y - pis->current_point.y) > 0.01)
+        return(false);
 
     size = pdf_calculate_text_size(pis, pdfont, &font->FontMatrix, &smat, &tmat, font, pdev);
 
@@ -710,7 +714,7 @@ bool pdf_compare_text_state_for_charpath(pdf_text_state_t *pts, gx_device_pdf *p
      * the text state.
      */
     if(size != pts->in.size)
-	return(false);
+        return(false);
 
     return(true);
 }
@@ -735,54 +739,54 @@ void pdf_set_text_render_mode(pdf_text_state_t *pts, int mode)
 int pdf_modify_text_render_mode(pdf_text_state_t *pts, int render_mode)
 {
     switch (pts->in.render_mode) {
-	case 0:
-	    if (render_mode == 1) {
-		pts->in.render_mode = 2;
-		return(1);
-	    }
-	    break;
-	case 1:
-	    if (render_mode == 1) 
-		return(1);
-	    break;
-	case 2:
-	    if (render_mode == 1) 
-		return(1);
-	    break;
-	case 3:
-	    if (render_mode == 1) {
-	        pts->in.render_mode = 1;
-		return(1);
-	    }
-	    break;
-	case 4:
-	    if (render_mode == 1) {
-	        pts->in.render_mode = 6;
-		return(1);
-	    }
-	    break;
-	case 5:
-	    if (render_mode == 1) 
-		return(1);
-	    break;
+        case 0:
+            if (render_mode == 1) {
+                pts->in.render_mode = 2;
+                return(1);
+            }
+            break;
+        case 1:
+            if (render_mode == 1)
+                return(1);
+            break;
+        case 2:
+            if (render_mode == 1)
+                return(1);
+            break;
+        case 3:
+            if (render_mode == 1) {
+                pts->in.render_mode = 1;
+                return(1);
+            }
+            break;
+        case 4:
+            if (render_mode == 1) {
+                pts->in.render_mode = 6;
+                return(1);
+            }
+            break;
+        case 5:
+            if (render_mode == 1)
+                return(1);
+            break;
         case 6:
-	    if (render_mode == 1) 
-		return(1);
-	    break;
-	case 7:
-	    if (render_mode == 1) {
-	        pts->in.render_mode = 5;
-		return(1);
-	    }
-	    break;
-	default:
-	    break;
+            if (render_mode == 1)
+                return(1);
+            break;
+        case 7:
+            if (render_mode == 1) {
+                pts->in.render_mode = 5;
+                return(1);
+            }
+            break;
+        default:
+            break;
     }
     return(0);
 }
 
-int pdf_set_PaintType0_params (gx_device_pdf *pdev, gs_imager_state *pis, float size, 
-			       double scaled_width, const pdf_text_state_values_t *ptsv)
+int pdf_set_PaintType0_params (gx_device_pdf *pdev, gs_imager_state *pis, float size,
+                               double scaled_width, const pdf_text_state_values_t *ptsv)
 {
     pdf_text_state_t *pts = pdev->text->text_state;
     double saved_width = pis->line_params.half_width;
@@ -804,22 +808,22 @@ int pdf_set_PaintType0_params (gx_device_pdf *pdev, gs_imager_state *pis, float
      * (new text) Tj
      */
     if (pts->buffer.count_chars > 0) {
-	if (pts->PaintType0Width != scaled_width) {
-	    pis->line_params.half_width = scaled_width / 2;
-	    code = pdf_set_text_state_values(pdev, ptsv);
-	    if (code < 0)
-		return code;
-	    if (pdev->text->text_state->in.render_mode == ptsv->render_mode){
-		code = pdf_prepare_stroke(pdev, pis);
-		if (code >= 0) 
-		    code = gdev_vector_prepare_stroke((gx_device_vector *)pdev,
-					      pis, NULL, NULL, 1);
-	    }
-	    if (code < 0)
-		return code;
-	    pis->line_params.half_width = saved_width;
-	    pts->PaintType0Width = scaled_width;
-	}
+        if (pts->PaintType0Width != scaled_width) {
+            pis->line_params.half_width = scaled_width / 2;
+            code = pdf_set_text_state_values(pdev, ptsv);
+            if (code < 0)
+                return code;
+            if (pdev->text->text_state->in.render_mode == ptsv->render_mode){
+                code = pdf_prepare_stroke(pdev, pis);
+                if (code >= 0)
+                    code = gdev_vector_prepare_stroke((gx_device_vector *)pdev,
+                                              pis, NULL, NULL, 1);
+            }
+            if (code < 0)
+                return code;
+            pis->line_params.half_width = saved_width;
+            pts->PaintType0Width = scaled_width;
+        }
     }
     return 0;
 }
