@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsfcid.c,v 1.2 2010/07/10 22:02:26 Arabidopsis Exp $ */
+/* $Id$ */
 /* Support for CID-keyed fonts */
 #include "memory_.h"
 #include "gx.h"
@@ -32,8 +32,8 @@ ENUM_PTRS_WITH(font_cid0_enum_ptrs, gs_font_cid0 *pfcid0)
 {
     index -= 2;
     if (index < st_gs_font_cid_data_num_ptrs)
-	return ENUM_USING(st_gs_font_cid_data, &pfcid0->cidata.common,
-			  sizeof(gs_font_cid_data), index);
+        return ENUM_USING(st_gs_font_cid_data, &pfcid0->cidata.common,
+                          sizeof(gs_font_cid_data), index);
     ENUM_PREFIX(st_gs_font_base, st_gs_font_cid_data_num_ptrs);
 }
 ENUM_PTR(0, gs_font_cid0, cidata.FDArray);
@@ -43,7 +43,7 @@ static
 RELOC_PTRS_WITH(font_cid0_reloc_ptrs, gs_font_cid0 *pfcid0);
     RELOC_PREFIX(st_gs_font_base);
     RELOC_USING(st_gs_font_cid_data, &pfcid0->cidata.common,
-		sizeof(st_gs_font_cid_data));
+                sizeof(st_gs_font_cid_data));
     RELOC_VAR(pfcid0->cidata.FDArray);
     RELOC_VAR(pfcid0->cidata.proc_data);
 RELOC_PTRS_END
@@ -52,8 +52,8 @@ static
 ENUM_PTRS_WITH(font_cid1_enum_ptrs, gs_font_cid1 *pfcid1)
 {
     if (index < st_cid_system_info_num_ptrs)
-	return ENUM_USING(st_cid_system_info, &pfcid1->cidata.CIDSystemInfo,
-			  sizeof(st_cid_system_info), index);
+        return ENUM_USING(st_cid_system_info, &pfcid1->cidata.CIDSystemInfo,
+                          sizeof(st_cid_system_info), index);
     ENUM_PREFIX(st_gs_font_base, st_cid_system_info_num_ptrs);
 }
 ENUM_PTRS_END
@@ -61,17 +61,17 @@ static
 RELOC_PTRS_WITH(font_cid1_reloc_ptrs, gs_font_cid1 *pfcid1);
     RELOC_PREFIX(st_gs_font_base);
     RELOC_USING(st_cid_system_info, &pfcid1->cidata.CIDSystemInfo,
-		sizeof(st_cid_system_info));
+                sizeof(st_cid_system_info));
 RELOC_PTRS_END
 public_st_gs_font_cid2();
 static
 ENUM_PTRS_WITH(font_cid2_enum_ptrs, gs_font_cid2 *pfcid2)
 {
     if (index < st_gs_font_cid2_own_ptrs)
-	ENUM_PTR(0, gs_font_cid2, subst_CID_on_WMode);
+        ENUM_PTR(0, gs_font_cid2, subst_CID_on_WMode);
     if (index < st_gs_font_cid_data_num_ptrs + st_gs_font_cid2_own_ptrs)
-	return ENUM_USING(st_gs_font_cid_data, &pfcid2->cidata.common,
-			  sizeof(gs_font_cid_data), index - st_gs_font_cid2_own_ptrs);
+        return ENUM_USING(st_gs_font_cid_data, &pfcid2->cidata.common,
+                          sizeof(gs_font_cid_data), index - st_gs_font_cid2_own_ptrs);
     ENUM_PREFIX(st_gs_font_type42, st_gs_font_cid_data_num_ptrs + st_gs_font_cid2_own_ptrs);
 }
 ENUM_PTRS_END
@@ -79,7 +79,7 @@ static
 RELOC_PTRS_WITH(font_cid2_reloc_ptrs, gs_font_cid2 *pfcid2);
     RELOC_PREFIX(st_gs_font_type42);
     RELOC_USING(st_gs_font_cid_data, &pfcid2->cidata.common,
-		sizeof(st_gs_font_cid_data));
+                sizeof(st_gs_font_cid_data));
     RELOC_VAR(pfcid2->subst_CID_on_WMode);
 RELOC_PTRS_END
 
@@ -130,7 +130,7 @@ bool
 cid_system_info_is_null(const gs_cid_system_info_t *pcidsi)
 {
     return (pcidsi->Registry.size == 0 && pcidsi->Ordering.size == 0 &&
-	    pcidsi->Supplement == 0);
+            pcidsi->Supplement == 0);
 }
 
 /*
@@ -142,35 +142,35 @@ gs_font_cid_system_info(const gs_font *pfont)
 {
     switch (pfont->FontType) {
     case ft_CID_encrypted:
-	return &((const gs_font_cid0 *)pfont)->cidata.common.CIDSystemInfo;
+        return &((const gs_font_cid0 *)pfont)->cidata.common.CIDSystemInfo;
     case ft_CID_user_defined:
-	return &((const gs_font_cid1 *)pfont)->cidata.CIDSystemInfo;
+        return &((const gs_font_cid1 *)pfont)->cidata.CIDSystemInfo;
     case ft_CID_TrueType:
-	return &((const gs_font_cid2 *)pfont)->cidata.common.CIDSystemInfo;
+        return &((const gs_font_cid2 *)pfont)->cidata.common.CIDSystemInfo;
     default:
-	return 0;
+        return 0;
     }
 }
 
 /*
  * Check CIDSystemInfo compatibility.
  */
-bool 
-gs_is_CIDSystemInfo_compatible(const gs_cid_system_info_t *info0, 
-			       const gs_cid_system_info_t *info1)
+bool
+gs_is_CIDSystemInfo_compatible(const gs_cid_system_info_t *info0,
+                               const gs_cid_system_info_t *info1)
 {
     if (info0 == NULL || info1 == NULL)
-	return false;
+        return false;
     if (info0->Registry.size != info1->Registry.size)
-	return false;
+        return false;
     if (info0->Ordering.size !=	info1->Ordering.size)
-	return false;
-    if (memcmp(info0->Registry.data, info1->Registry.data, 
-	       info0->Registry.size))
-	return false;
+        return false;
+    if (memcmp(info0->Registry.data, info1->Registry.data,
+               info0->Registry.size))
+        return false;
     if (memcmp(info0->Ordering.data, info1->Ordering.data,
-	       info0->Ordering.size))
-	return false;
+               info0->Ordering.size))
+        return false;
     return true;
 }
 
@@ -181,25 +181,25 @@ gs_is_CIDSystemInfo_compatible(const gs_cid_system_info_t *info0,
 font_proc_enumerate_glyph(gs_font_cid0_enumerate_glyph); /* check prototype */
 int
 gs_font_cid0_enumerate_glyph(gs_font *font, int *pindex,
-			     gs_glyph_space_t ignore_glyph_space,
-			     gs_glyph *pglyph)
+                             gs_glyph_space_t ignore_glyph_space,
+                             gs_glyph *pglyph)
 {
     gs_font_cid0 *const pfont = (gs_font_cid0 *)font;
 
     while (*pindex < pfont->cidata.common.CIDCount) {
-	gs_glyph_data_t gdata;
-	int fidx;
-	gs_glyph glyph = (gs_glyph)(gs_min_cid_glyph + (*pindex)++);
-	int code;
+        gs_glyph_data_t gdata;
+        int fidx;
+        gs_glyph glyph = (gs_glyph)(gs_min_cid_glyph + (*pindex)++);
+        int code;
 
-	gdata.memory = pfont->memory;
-	code = pfont->cidata.glyph_data((gs_font_base *)pfont, glyph,
-					    &gdata, &fidx);
-	if (code < 0 || gdata.bits.size == 0)
-	    continue;
-	*pglyph = glyph;
-	gs_glyph_data_free(&gdata, "gs_font_cid0_enumerate_glyphs");
-	return 0;
+        gdata.memory = pfont->memory;
+        code = pfont->cidata.glyph_data((gs_font_base *)pfont, glyph,
+                                            &gdata, &fidx);
+        if (code < 0 || gdata.bits.size == 0)
+            continue;
+        *pglyph = glyph;
+        gs_glyph_data_free(&gdata, "gs_font_cid0_enumerate_glyphs");
+        return 0;
     }
     *pindex = 0;
     return 0;
@@ -212,7 +212,9 @@ gs_cid0_indexed_font(const gs_font *font, int fidx)
     gs_font_cid0 *const pfont = (gs_font_cid0 *)font;
 
     if (font->FontType != ft_CID_encrypted) {
-	eprintf1("Unexpected font type: %d\n", font->FontType);
+        emprintf1(font->memory,
+                  "Unexpected font type: %d\n",
+                  font->FontType);
         return 0;
     }
     return (const gs_font*) (pfont->cidata.FDArray[fidx]);
@@ -226,11 +228,13 @@ gs_cid0_has_type2(const gs_font *font)
     int i;
 
     if (font->FontType != ft_CID_encrypted) {
-	eprintf1("Unexpected font type: %d\n", font->FontType);
+        emprintf1(font->memory,
+                  "Unexpected font type: %d\n",
+                  font->FontType);
         return false;
     }
     for (i = 0; i < pfont->cidata.FDArray_size; i++)
-	if (((const gs_font *)pfont->cidata.FDArray[i])->FontType == ft_encrypted2)
-	    return true;
+        if (((const gs_font *)pfont->cidata.FDArray[i])->FontType == ft_encrypted2)
+            return true;
     return false;
 }
