@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsiodev.c,v 1.2 2010/07/10 22:02:23 Arabidopsis Exp $ */
+/* $Id$ */
 /* IODevice implementation for Ghostscript */
 #include "errno_.h"
 #include "string_.h"
@@ -28,16 +28,12 @@
 /* Import the IODevice table from gconf.c. */
 extern_gx_io_device_table();
 
-/* Define a table of local copies of the IODevices, */
-/* allocated at startup.  This just postpones the day of reckoning.... */
-static gx_io_device **io_device_table;
-
 private_st_io_device();
 gs_private_st_ptr(st_io_device_ptr, gx_io_device *, "gx_io_device *",
-		  iodev_ptr_enum_ptrs, iodev_ptr_reloc_ptrs);
-gs_private_st_element(st_io_device_ptr_element, gx_io_device *,
+                  iodev_ptr_enum_ptrs, iodev_ptr_reloc_ptrs);
+gs_private_st_element_final(st_io_device_ptr_element, gx_io_device *,
       "gx_io_device *[]", iodev_ptr_elt_enum_ptrs, iodev_ptr_elt_reloc_ptrs,
-		      st_io_device_ptr);
+                      st_io_device_ptr,gs_iodev_finalize);
 
 /* Define the OS (%os%) device. */
 iodev_proc_fopen(iodev_os_fopen);
@@ -65,45 +61,65 @@ int
 gs_iodev_init(gs_memory_t * mem)
 {				/* Make writable copies of all IODevices. */
     gx_io_device **table =
-	gs_alloc_struct_array(mem, gx_io_device_table_count,
-			      gx_io_device *, &st_io_device_ptr_element,
-			      "gs_iodev_init(table)");
+        gs_alloc_struct_array(mem, gx_io_device_table_count + 1,
+                              gx_io_device *, &st_io_device_ptr_element,
+                              "gs_iodev_init(table)");
+    gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
     int i, j;
     int code = 0;
 
-    if (table == 0)
-	return_error(gs_error_VMerror);
+    if ((table == NULL) || (libctx == NULL))
+        return_error(gs_error_VMerror);
     for (i = 0; i < gx_io_device_table_count; ++i) {
-	gx_io_device *iodev =
-	    gs_alloc_struct(mem, gx_io_device, &st_io_device,
-			    "gs_iodev_init(iodev)");
+        gx_io_device *iodev =
+            gs_alloc_struct(mem, gx_io_device, &st_io_device,
+                            "gs_iodev_init(iodev)");
 
-	if (iodev == 0)
-	    goto fail;
-	table[i] = iodev;
-	memcpy(table[i], gx_io_device_table[i], sizeof(gx_io_device));
+        if (iodev == 0)
+            goto fail;
+        table[i] = iodev;
+        memcpy(table[i], gx_io_device_table[i], sizeof(gx_io_device));
     }
-    io_device_table = table;
-    code = gs_register_struct_root(mem, NULL, (void **)&io_device_table,
-				   "io_device_table");
+    /* FIXME: we allocate an extra pointer in the array and stuff
+     * the memory context in there, as we need access to it in the
+     * finalize method below. We get away it because everywhere that
+     * searches the table uses gx_io_device_table_count, rather than
+     * the actuall allocated length.
+     * A less naff solution would be preferable!
+     */
+    table[gx_io_device_table_count] = (gx_io_device *) mem;
+    libctx->io_device_table = table;
+    code = gs_register_struct_root(mem, NULL,
+                                   (void **)&libctx->io_device_table,
+                                   "io_device_table");
     if (code < 0)
-	goto fail;
+        goto fail;
     /* Run the one-time initialization of each IODevice. */
     for (j = 0; j < gx_io_device_table_count; ++j)
-	if ((code = (table[j]->procs.init)(table[j], mem)) < 0)
-	    goto f2;
+        if ((code = (table[j]->procs.init)(table[j], mem)) < 0)
+            goto f2;
     return 0;
  f2:
     /****** CAN'T FIND THE ROOT ******/
     /*gs_unregister_root(mem, root, "io_device_table");*/
  fail:
     for (; i >= 0; --i)
-	gs_free_object(mem, table[i - 1], "gs_iodev_init(iodev)");
+        gs_free_object(mem, table[i - 1], "gs_iodev_init(iodev)");
     gs_free_object(mem, table, "gs_iodev_init(table)");
-    io_device_table = 0;
+    libctx->io_device_table = 0;
     return (code < 0 ? code : gs_note_error(gs_error_VMerror));
 }
 
+static void
+gs_iodev_finalize(void *vptr)
+{
+    gx_io_device **table = vptr;
+    gs_memory_t * mem = ((gs_memory_t *)table[gx_io_device_table_count]);
+
+    mem->gs_lib_ctx->io_device_table = NULL;
+}
+
+
 /* ------ Default (unimplemented) IODevice procedures ------ */
 
 int
@@ -114,21 +130,21 @@ iodev_no_init(gx_io_device * iodev, gs_memory_t * mem)
 
 int
 iodev_no_open_device(gx_io_device * iodev, const char *access, stream ** ps,
-		     gs_memory_t * mem)
+                     gs_memory_t * mem)
 {
     return_error(gs_error_invalidfileaccess);
 }
 
 int
 iodev_no_open_file(gx_io_device * iodev, const char *fname, uint namelen,
-		   const char *access, stream ** ps, gs_memory_t * mem)
+                   const char *access, stream ** ps, gs_memory_t * mem)
 {
     return_error(gs_error_invalidfileaccess);
 }
 
 int
 iodev_no_fopen(gx_io_device * iodev, const char *fname, const char *access,
-	       FILE ** pfile, char *rfname, uint rnamelen)
+               FILE ** pfile, char *rfname, uint rnamelen)
 {
     return_error(gs_error_invalidfileaccess);
 }
@@ -159,7 +175,7 @@ iodev_no_file_status(gx_io_device * iodev, const char *fname, struct stat *pstat
 
 file_enum *
 iodev_no_enumerate_files(gx_io_device * iodev, const char *pat, uint patlen,
-			 gs_memory_t * memory)
+                         gs_memory_t * memory)
 {
     return NULL;
 }
@@ -181,14 +197,14 @@ iodev_no_put_params(gx_io_device * iodev, gs_param_list * plist)
 /* The fopen routine is exported for %null. */
 int
 iodev_os_fopen(gx_io_device * iodev, const char *fname, const char *access,
-	       FILE ** pfile, char *rfname, uint rnamelen)
+               FILE ** pfile, char *rfname, uint rnamelen)
 {
     errno = 0;
     *pfile = gp_fopen(fname, access);
     if (*pfile == NULL)
-	return_error(gs_fopen_errno_to_code(errno));
+        return_error(gs_fopen_errno_to_code(errno));
     if (rfname != NULL && rfname != fname)
-	strcpy(rfname, fname);
+        strcpy(rfname, fname);
     return 0;
 }
 
@@ -221,7 +237,7 @@ os_status(gx_io_device * iodev, const char *fname, struct stat *pstat)
 
 static file_enum *
 os_enumerate(gx_io_device * iodev, const char *pat, uint patlen,
-	     gs_memory_t * mem)
+             gs_memory_t * mem)
 {
     return gp_enumerate_files_init(pat, patlen, mem);
 }
@@ -244,18 +260,18 @@ os_get_params(gx_io_device * iodev, gs_param_list * plist)
     Free = LogicalSize * 3 / 4;			/* about 1.5 Gb */
 
     if (
-	(code = param_write_bool(plist, "HasNames", &btrue)) < 0 ||
-	(code = param_write_int(plist, "BlockSize", &BlockSize)) < 0 ||
-	(code = param_write_long(plist, "Free", &Free)) < 0 ||
-	(code = param_write_int(plist, "InitializeAction", &i0)) < 0 ||
-	(code = param_write_bool(plist, "Mounted", &btrue)) < 0 ||
-	(code = param_write_bool(plist, "Removable", &bfalse)) < 0 ||
-	(code = param_write_bool(plist, "Searchable", &btrue)) < 0 ||
-	(code = param_write_int(plist, "SearchOrder", &i2)) < 0 ||
-	(code = param_write_bool(plist, "Writeable", &btrue)) < 0 ||
-	(code = param_write_long(plist, "LogicalSize", &LogicalSize)) < 0
-	)
-	return code;
+        (code = param_write_bool(plist, "HasNames", &btrue)) < 0 ||
+        (code = param_write_int(plist, "BlockSize", &BlockSize)) < 0 ||
+        (code = param_write_long(plist, "Free", &Free)) < 0 ||
+        (code = param_write_int(plist, "InitializeAction", &i0)) < 0 ||
+        (code = param_write_bool(plist, "Mounted", &btrue)) < 0 ||
+        (code = param_write_bool(plist, "Removable", &bfalse)) < 0 ||
+        (code = param_write_bool(plist, "Searchable", &btrue)) < 0 ||
+        (code = param_write_int(plist, "SearchOrder", &i2)) < 0 ||
+        (code = param_write_bool(plist, "Writeable", &btrue)) < 0 ||
+        (code = param_write_long(plist, "LogicalSize", &LogicalSize)) < 0
+        )
+        return code;
     return 0;
 }
 
@@ -263,28 +279,32 @@ os_get_params(gx_io_device * iodev, gs_param_list * plist)
 
 /* Get the N'th IODevice from the known device table. */
 gx_io_device *
-gs_getiodevice(int index)
+gs_getiodevice(const gs_memory_t *mem, int index)
 {
-    if (!io_device_table || index < 0 || index >= gx_io_device_table_count)
-	return 0;		/* index out of range */
-    return io_device_table[index];
+    gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
+
+    if (libctx == NULL || libctx->io_device_table == NULL ||
+        index < 0      || index >= gx_io_device_table_count)
+        return 0;		/* index out of range */
+    return libctx->io_device_table[index];
 }
 
 /* Look up an IODevice name. */
 /* The name may be either %device or %device%. */
 gx_io_device *
-gs_findiodevice(const byte * str, uint len)
+gs_findiodevice(const gs_memory_t *mem, const byte * str, uint len)
 {
     int i;
+    gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
 
     if (len > 1 && str[len - 1] == '%')
-	len--;
+        len--;
     for (i = 0; i < gx_io_device_table_count; ++i) {
-	gx_io_device *iodev = io_device_table[i];
-	const char *dname = iodev->dname;
+        gx_io_device *iodev = libctx->io_device_table[i];
+        const char *dname = iodev->dname;
 
-	if (dname && strlen(dname) == len + 1 && !memcmp(str, dname, len))
-	    return iodev;
+        if (dname && strlen(dname) == len + 1 && !memcmp(str, dname, len))
+            return iodev;
     }
     return 0;
 }
@@ -301,7 +321,7 @@ gs_getdevparams(gx_io_device * iodev, gs_param_list * plist)
     param_string_from_string(ts, iodev->dtype);
     code = param_write_name(plist, "Type", &ts);
     if (code < 0)
-	return code;
+        return code;
     return (*iodev->procs.get_params) (iodev, plist);
 }
 
@@ -320,36 +340,36 @@ gs_fopen_errno_to_code(int eno)
     /* We try to cover as many variations as we know about. */
     switch (eno) {
 #ifdef ENOENT
-	case ENOENT:
-	    return_error(gs_error_undefinedfilename);
+        case ENOENT:
+            return_error(gs_error_undefinedfilename);
 #endif
 #ifdef ENOFILE
 #  ifndef ENOENT
 #    define ENOENT ENOFILE
 #  endif
 #  if ENOFILE != ENOENT
-	case ENOFILE:
-	    return_error(gs_error_undefinedfilename);
+        case ENOFILE:
+            return_error(gs_error_undefinedfilename);
 #  endif
 #endif
 #ifdef ENAMETOOLONG
-	case ENAMETOOLONG:
-	    return_error(gs_error_undefinedfilename);
+        case ENAMETOOLONG:
+            return_error(gs_error_undefinedfilename);
 #endif
 #ifdef EACCES
-	case EACCES:
-	    return_error(gs_error_invalidfileaccess);
+        case EACCES:
+            return_error(gs_error_invalidfileaccess);
 #endif
 #ifdef EMFILE
-	case EMFILE:
-	    return_error(gs_error_limitcheck);
+        case EMFILE:
+            return_error(gs_error_limitcheck);
 #endif
 #ifdef ENFILE
-	case ENFILE:
-	    return_error(gs_error_limitcheck);
+        case ENFILE:
+            return_error(gs_error_limitcheck);
 #endif
-	default:
-	    return_error(gs_error_ioerror);
+        default:
+            return_error(gs_error_ioerror);
     }
 }
 
@@ -365,7 +385,7 @@ struct gs_file_enum_s {
 };
 
 gs_private_st_ptrs1(st_gs_file_enum, gs_file_enum, "gs_file_enum",
-		    gs_file_enum_enum_ptrs, gs_file_enum_reloc_ptrs, pfile_enum);
+                    gs_file_enum_enum_ptrs, gs_file_enum_reloc_ptrs, pfile_enum);
 
 file_enum *
 gs_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
@@ -377,23 +397,23 @@ gs_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
     int code = 0;
 
     /* Get the iodevice */
-    code = gs_parse_file_name(&pfn, pat, patlen);
+    code = gs_parse_file_name(&pfn, pat, patlen, mem);
     if (code < 0)
-	return NULL;
-    iodev = (pfn.iodev == NULL) ? iodev_default : pfn.iodev;
+        return NULL;
+    iodev = (pfn.iodev == NULL) ? iodev_default(mem) : pfn.iodev;
 
     /* Check for several conditions that just cause us to return success */
     if (pfn.len == 0 || iodev->procs.enumerate_files == iodev_no_enumerate_files) {
         return NULL;	/* no pattern, or device not found -- just return */
     }
     pfen = iodev->procs.enumerate_files(iodev, (const char *)pfn.fname,
-    		pfn.len, mem);
+                pfn.len, mem);
     if (pfen == 0)
-	return NULL;
+        return NULL;
     pgs_file_enum = gs_alloc_struct(mem, gs_file_enum, &st_gs_file_enum,
-			   "gs_enumerate_files_init");
+                           "gs_enumerate_files_init");
     if (pgs_file_enum == 0)
-	return NULL;
+        return NULL;
     pgs_file_enum->memory = mem;
     pgs_file_enum->piodev = iodev;
     pgs_file_enum->pfile_enum = pfen;
@@ -406,20 +426,20 @@ gs_enumerate_files_next(file_enum * pfen, char *ptr, uint maxlen)
 {
     gs_file_enum *pgs_file_enum = (gs_file_enum *)pfen;
     int iodev_name_len = pgs_file_enum->prepend_iodev_name ?
-			strlen(pgs_file_enum->piodev->dname) : 0;
+                        strlen(pgs_file_enum->piodev->dname) : 0;
     uint return_len;
 
     if (iodev_name_len > maxlen)
-	return maxlen + 1;	/* signal overflow error */
+        return maxlen + 1;	/* signal overflow error */
     if (iodev_name_len > 0)
-	memcpy(ptr, pgs_file_enum->piodev->dname, iodev_name_len);
+        memcpy(ptr, pgs_file_enum->piodev->dname, iodev_name_len);
     return_len = pgs_file_enum->piodev->procs.enumerate_next(pgs_file_enum->pfile_enum,
-				ptr + iodev_name_len, maxlen - iodev_name_len);
+                                ptr + iodev_name_len, maxlen - iodev_name_len);
     if (return_len == ~0) {
         gs_memory_t *mem = pgs_file_enum->memory;
 
         gs_free_object(mem, pgs_file_enum, "gs_enumerate_files_close");
-	return ~0;
+        return ~0;
     }
     return return_len+iodev_name_len;
 }
