@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsiparm4.h,v 1.2 2010/07/10 22:02:22 Arabidopsis Exp $ */
+/* $Id$ */
 /* ImageType 4 image parameter definition */
 
 #ifndef gsiparm4_INCLUDED
@@ -37,6 +37,13 @@ typedef struct gs_image4_s {
      */
     bool MaskColor_is_range;
     uint MaskColor[GS_IMAGE_MAX_COMPONENTS * 2];
+    /*
+     * Define the parent image type that gave rise to this.
+     * Used to avoid the use of mixed halftoning methods
+     * between images and their masks, which
+     * can cause misalignment issues in pixel replications.
+     */
+    gs_image_parent_t image_parent_type;
 } gs_image4_t;
 
 #define private_st_gs_image4()	/* in gximage4.c */\
