@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsptype2.c,v 1.2 2010/07/10 22:02:23 Arabidopsis Exp $ */
+/* $Id$ */
 /* PatternType 2 implementation */
 #include "gx.h"
 #include "gserrors.h"
@@ -27,6 +27,7 @@
 #include "gzpath.h"
 #include "gzcpath.h"
 #include "gzstate.h"
+#include "gxdevsop.h"
 
 /* GC descriptors */
 private_st_pattern2_template();
@@ -115,13 +116,12 @@ gs_pattern2_set_shfill(gs_client_color * pcc)
     gs_pattern2_instance_t *pinst;
 
     if (pcc->pattern->type != &gs_pattern2_type)
-	return_error(gs_error_unregistered); /* Must not happen. */
+        return_error(gs_error_unregistered); /* Must not happen. */
     pinst = (gs_pattern2_instance_t *)pcc->pattern;
     pinst->shfill = true;
     return 0;
 }
 
-
 /* ---------------- Rendering ---------------- */
 
 /* GC descriptor */
@@ -144,7 +144,7 @@ const gx_device_color_type_t gx_dc_pattern2 = {
     gx_dc_ht_get_phase,
     gx_dc_pattern2_load, gx_dc_pattern2_fill_rectangle,
     gx_dc_default_fill_masked, gx_dc_pattern2_equal,
-    gx_dc_cannot_write, gx_dc_cannot_read, 
+    gx_dc_cannot_write, gx_dc_cannot_read,
     gx_dc_pattern_get_nonzero_comps
 };
 
@@ -207,30 +207,30 @@ gs_pattern2_set_color(const gs_client_color * pcc, gs_state * pgs)
 
 /* Fill a rectangle with a PatternType 2 color. */
 /* WARNING: This function doesn't account the shading BBox
-   to allow the clipent to optimize the clipping 
-   with changing the order of clip patrhs and rects.
-   The client must clip with the shading BBOx before calling this function. */
+   to allow the clipent to optimize the clipping
+   with changing the order of clip paths and rects.
+   The client must clip with the shading BBox before calling this function. */
 static int
 gx_dc_pattern2_fill_rectangle(const gx_device_color * pdevc, int x, int y,
                               int w, int h, gx_device * dev,
                               gs_logical_operation_t lop,
                               const gx_rop_source_t * source)
 {
-    if (dev_proc(dev, pattern_manage)(dev, gs_no_id, NULL, pattern_manage__is_cpath_accum)) {
-	/* Performing a conversion of imagemask into a clipping path.
-	   Fall back to the device procedure. */
-	return dev_proc(dev, fill_rectangle)(dev, x, y, w, h, (gx_color_index)0/*any*/);
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_pattern_is_cpath_accum, NULL, 0)) {
+        /* Performing a conversion of imagemask into a clipping path.
+           Fall back to the device procedure. */
+        return dev_proc(dev, fill_rectangle)(dev, x, y, w, h, (gx_color_index)0/*any*/);
     } else {
-	gs_fixed_rect rect;
-	gs_pattern2_instance_t *pinst =
-	    (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+        gs_fixed_rect rect;
+        gs_pattern2_instance_t *pinst =
+            (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
 
-	rect.p.x = int2fixed(x);
-	rect.p.y = int2fixed(y);
-	rect.q.x = int2fixed(x + w);
-	rect.q.y = int2fixed(y + h);
-	return gs_shading_do_fill_rectangle(pinst->template.Shading, &rect, dev,
-				    (gs_imager_state *)pinst->saved, !pinst->shfill);
+        rect.p.x = int2fixed(x);
+        rect.p.y = int2fixed(y);
+        rect.q.x = int2fixed(x + w);
+        rect.q.y = int2fixed(y + h);
+        return gs_shading_do_fill_rectangle(pinst->template.Shading, &rect, dev,
+                                    (gs_imager_state *)pinst->saved, !pinst->shfill);
     }
 }
 
@@ -253,7 +253,7 @@ gx_dc_pattern2_equal(const gx_device_color * pdevc1,
  */
 static void
 gx_dc_pattern2_save_dc(
-    const gx_device_color * pdevc, 
+    const gx_device_color * pdevc,
     gx_device_color_saved * psdc )
 {
     gs_pattern2_instance_t * pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
@@ -267,16 +267,16 @@ gx_dc_pattern2_save_dc(
 /* This is just a bridge to an old code. */
 int
 gx_dc_pattern2_shade_bbox_transform2fixed(const gs_rect * rect, const gs_imager_state * pis,
-			   gs_fixed_rect * rfixed)
+                           gs_fixed_rect * rfixed)
 {
     gs_rect dev_rect;
     int code = gs_bbox_transform(rect, &ctm_only(pis), &dev_rect);
 
     if (code >= 0) {
-	rfixed->p.x = float2fixed(dev_rect.p.x);
-	rfixed->p.y = float2fixed(dev_rect.p.y);
-	rfixed->q.x = float2fixed(dev_rect.q.x);
-	rfixed->q.y = float2fixed(dev_rect.q.y);
+        rfixed->p.x = float2fixed(dev_rect.p.x);
+        rfixed->p.y = float2fixed(dev_rect.p.y);
+        rfixed->q.x = float2fixed(dev_rect.q.x);
+        rfixed->q.y = float2fixed(dev_rect.q.y);
     }
     return code;
 }
@@ -290,11 +290,11 @@ gx_dc_pattern2_get_bbox(const gx_device_color * pdevc, gs_fixed_rect *bbox)
     int code;
 
     if (!pinst->template.Shading->params.have_BBox)
-	return 0;
+        return 0;
     code = gx_dc_pattern2_shade_bbox_transform2fixed(
-		&pinst->template.Shading->params.BBox, (gs_imager_state *)pinst->saved, bbox);
+                &pinst->template.Shading->params.BBox, (gs_imager_state *)pinst->saved, bbox);
     if (code < 0)
-	return code;
+        return code;
     return 1;
 }
 
@@ -315,11 +315,11 @@ gx_dc_shading_path_add_box(gx_path *ppath, const gx_device_color * pdevc)
     const gs_shading_t *psh = pinst->template.Shading;
 
     if (!psh->params.have_BBox)
-	return_error(gs_error_unregistered); /* Do not call in this case. */
+        return_error(gs_error_unregistered); /* Do not call in this case. */
     else {
-	gs_state *pis = pinst->saved;
+        gs_state *pis = pinst->saved;
 
-	return gs_shading_path_add_box(ppath, &psh->params.BBox, &pis->ctm);
+        return gs_shading_path_add_box(ppath, &psh->params.BBox, &pis->ctm);
     }
 }
 
@@ -329,7 +329,7 @@ gx_dc_pattern2_clip_with_bbox(const gx_device_color * pdevc, gx_device * pdev,
                               gx_clip_path *cpath_local, const gx_clip_path **ppcpath1)
 {
     if (gx_dc_is_pattern2_color(pdevc) && gx_dc_pattern2_color_has_bbox(pdevc) &&
-            (*dev_proc(pdev, pattern_manage))(pdev, gs_no_id, NULL, pattern_manage__shading_area) == 0) {
+            (*dev_proc(pdev, dev_spec_op))(pdev, gxdso_pattern_shading_area, NULL, 0) == 0) {
         gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
         gx_path box_path;
         gs_memory_t *mem = (*ppcpath1 != NULL ? (*ppcpath1)->path.memory : pdev->memory);
@@ -354,26 +354,26 @@ gx_dc_pattern2_clip_with_bbox(const gx_device_color * pdevc, gx_device * pdev,
 
 /* Intersect a clipping path a shading BBox. */
 int
-gx_dc_pattern2_clip_with_bbox_simple(const gx_device_color * pdevc, gx_device * pdev, 
-			      gx_clip_path *cpath_local)
+gx_dc_pattern2_clip_with_bbox_simple(const gx_device_color * pdevc, gx_device * pdev,
+                              gx_clip_path *cpath_local)
 {
     int code = 0;
 
     if (gx_dc_is_pattern2_color(pdevc) && gx_dc_pattern2_color_has_bbox(pdevc) &&
-	    (*dev_proc(pdev, pattern_manage))(pdev, gs_no_id, NULL, pattern_manage__shading_area) == 0) {
-	gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
-	gx_path box_path;
-	gs_memory_t *mem = cpath_local->path.memory;
+            (*dev_proc(pdev, dev_spec_op))(pdev, gxdso_pattern_shading_area, NULL, 0) == 0) {
+        gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+        gx_path box_path;
+        gs_memory_t *mem = cpath_local->path.memory;
 
-	gx_path_init_local(&box_path, mem);
-	code = gx_dc_shading_path_add_box(&box_path, pdevc);
-	if (code == gs_error_limitcheck) {
-	    /* Ignore huge BBox - bug 689027. */
-	    code = 0;
-	} else if (code >= 0) {
-	    code = gx_cpath_intersect(cpath_local, &box_path, gx_rule_winding_number, (gs_imager_state *)pinst->saved);
-	}
-	gx_path_free(&box_path, "gx_default_fill_path(path_bbox)");
+        gx_path_init_local(&box_path, mem);
+        code = gx_dc_shading_path_add_box(&box_path, pdevc);
+        if (code == gs_error_limitcheck) {
+            /* Ignore huge BBox - bug 689027. */
+            code = 0;
+        } else if (code >= 0) {
+            code = gx_cpath_intersect(cpath_local, &box_path, gx_rule_winding_number, (gs_imager_state *)pinst->saved);
+        }
+        gx_path_free(&box_path, "gx_default_fill_path(path_bbox)");
     }
     return code;
 }
@@ -383,36 +383,35 @@ int
 gx_dc_pattern2_is_rectangular_cell(const gx_device_color * pdevc, gx_device * pdev, gs_fixed_rect *rect)
 {
     if (gx_dc_is_pattern2_color(pdevc) && gx_dc_pattern2_color_has_bbox(pdevc) &&
-	    (*dev_proc(pdev, pattern_manage))(pdev, gs_no_id, NULL, pattern_manage__shading_area) == 0) {
-	gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
-	const gs_shading_t *psh = pinst->template.Shading;
-	gs_fixed_point p, q; 
+            (*dev_proc(pdev, dev_spec_op))(pdev, gxdso_pattern_shading_area, NULL, 0) == 0) {
+        gs_pattern2_instance_t *pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
+        const gs_shading_t *psh = pinst->template.Shading;
+        gs_fixed_point p, q;
 
-	if (is_xxyy(&ctm_only(pinst->saved)))
-	    if (psh->params.have_BBox) {
-		int code = gs_point_transform2fixed(&pinst->saved->ctm, 
-			    psh->params.BBox.p.x, psh->params.BBox.p.y, &p);
-		if (code < 0)
-		    return code;
-		code = gs_point_transform2fixed(&pinst->saved->ctm, 
-			    psh->params.BBox.q.x, psh->params.BBox.q.y, &q);
-		if (code < 0)
-		    return code;
-		if (p.x > q.x) {
-		    p.x ^= q.x; q.x ^= p.x; p.x ^= q.x;
-		}
-		if (p.y > q.y) {
-		    p.y ^= q.y; q.y ^= p.y; p.y ^= q.y;
-		}
-		rect->p = p;
-		rect->q = q;
-		return 1;
-	    }
+        if (is_xxyy(&ctm_only(pinst->saved)))
+            if (psh->params.have_BBox) {
+                int code = gs_point_transform2fixed(&pinst->saved->ctm,
+                            psh->params.BBox.p.x, psh->params.BBox.p.y, &p);
+                if (code < 0)
+                    return code;
+                code = gs_point_transform2fixed(&pinst->saved->ctm,
+                            psh->params.BBox.q.x, psh->params.BBox.q.y, &q);
+                if (code < 0)
+                    return code;
+                if (p.x > q.x) {
+                    p.x ^= q.x; q.x ^= p.x; p.x ^= q.x;
+                }
+                if (p.y > q.y) {
+                    p.y ^= q.y; q.y ^= p.y; p.y ^= q.y;
+                }
+                rect->p = p;
+                rect->q = q;
+                return 1;
+            }
     }
     return 0;
 }
 
-
 /* Get a shading color space. */
 const gs_color_space *
 gx_dc_pattern2_get_color_space(const gx_device_color * pdevc)
@@ -424,7 +423,6 @@ gx_dc_pattern2_get_color_space(const gx_device_color * pdevc)
     return psh->params.ColorSpace;
 }
 
-
 /* Check device color for a possibly self-overlapping shading. */
 bool
 gx_dc_pattern2_can_overlap(const gx_device_color *pdevc)
@@ -432,13 +430,13 @@ gx_dc_pattern2_can_overlap(const gx_device_color *pdevc)
     gs_pattern2_instance_t * pinst;
 
     if (pdevc->type != &gx_dc_pattern2)
-	return false;
+        return false;
     pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
     switch (pinst->template.Shading->head.type) {
-	case 3: case 6: case 7:
-	    return true;
-	default:
-	    return false;
+        case 3: case 6: case 7:
+            return true;
+        default:
+            return false;
     }
 }
 
@@ -449,7 +447,7 @@ bool gx_dc_pattern2_has_background(const gx_device_color *pdevc)
     const gs_shading_t *Shading;
 
     if (pdevc->type != &gx_dc_pattern2)
-	return false;
+        return false;
     pinst = (gs_pattern2_instance_t *)pdevc->ccolor.pattern;
     Shading = pinst->template.Shading;
     return !pinst->shfill && Shading->params.Background != NULL;
