@@ -1,19 +1,16 @@
 /*
     jbig2dec
-    
+
     Copyright (C) 2001-2002 Artifex Software, Inc.
-    
+
     This software is distributed under license and may not
     be copied, modified or distributed except as expressly
     authorized under the terms of the license contained in
     the file LICENSE in this distribution.
-                                                                                
-    For information on commercial licensing, go to
-    http://www.artifex.com/licensing/ or contact
-    Artifex Software, Inc.,  101 Lucas Valley Road #110,
-    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 
-    $Id: jbig2_image.h,v 1.6 2008/03/23 15:28:32 Arabidopsis Exp $
+    For further licensing information refer to http://artifex.com/ or
+    contact Artifex Software, Inc., 7 Mt. Lassen Drive - Suite A-134,
+    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
 
@@ -21,7 +18,7 @@
 #define _JBIG2_IMAGE_H
 
 int jbig2_image_get_pixel(Jbig2Image *image, int x, int y);
-int jbig2_image_set_pixel(Jbig2Image *image, int x, int y, int value);
+int jbig2_image_set_pixel(Jbig2Image *image, int x, int y, bool value);
 
 /* routines for dumping the image data in various formats */
 /* FIXME: should these be in the client instead? */
