@@ -82,7 +82,7 @@ jbig2_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_da
             index++;
             if (index >= ctx->max_page_index) {
                 /* grow the list */
-		ctx->pages = jbig2_realloc(ctx->allocator, ctx->pages,
+		ctx->pages = (Jbig2Page*)jbig2_realloc(ctx->allocator, ctx->pages,
 			(ctx->max_page_index <<= 2) * sizeof(Jbig2Page));
                 for (j=index; j < ctx->max_page_index; j++) {
                     ctx->pages[j].state = JBIG2_PAGE_FREE;
@@ -105,11 +105,11 @@ jbig2_page_info (Jbig2Ctx *ctx, Jbig2Segment *segment, const uint8_t *segment_da
     }
 
     /* 7.4.8.x */
-    page->width = jbig2_get_int32(segment_data);
-    page->height = jbig2_get_int32(segment_data + 4);
+    page->width = jbig2_get_uint32(segment_data);
+    page->height = jbig2_get_uint32(segment_data + 4);
 
-    page->x_resolution = jbig2_get_int32(segment_data + 8);
-    page->y_resolution = jbig2_get_int32(segment_data + 12);
+    page->x_resolution = jbig2_get_uint32(segment_data + 8);
+    page->y_resolution = jbig2_get_uint32(segment_data + 12);
     page->flags = segment_data[16];
 
     /* 7.4.8.6 */
@@ -291,10 +291,19 @@ Jbig2Image *jbig2_page_out(Jbig2Ctx *ctx)
     /* search for a completed page */
     for (index=0; index < ctx->max_page_index; index++) {
         if (ctx->pages[index].state == JBIG2_PAGE_COMPLETE) {
+            Jbig2Image *img = ctx->pages[index].image;
+            uint32_t page_number = ctx->pages[index].number;
             ctx->pages[index].state = JBIG2_PAGE_RETURNED;
-            jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, -1,
-                "page %d returned to the client", ctx->pages[index].number);
-            return jbig2_image_clone(ctx, ctx->pages[index].image);
+            if (img != NULL) {
+                jbig2_error(ctx, JBIG2_SEVERITY_DEBUG, -1,
+                            "page %d returned to the client", page_number);
+                return jbig2_image_clone(ctx, img);
+            } else {
+                jbig2_error(ctx, JBIG2_SEVERITY_WARNING, -1,
+                            "page %d returned with no associated image",
+                            page_number);
+                ; /* continue */
+            }
         }
     }
 
