@@ -4,24 +4,20 @@ exec tclsh "$0" "$@"
 
 #    Copyright (C) 1999, 2000, 2001 Aladdin Enterprises.  All rights reserved.
 # 
-# This file is part of GNU ghostscript
-#
-# GNU ghostscript is free software; you can redistribute it and/or
-# modify it under the terms of the version 2 of the GNU General Public
-# License as published by the Free Software Foundation.
-#
 # This software is provided AS-IS with no warranty, either express or
-# implied. That is, this program is distributed in the hope that it will 
-# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA, 02110-1301.
+# implied.
+# 
+# This software is distributed under license and may not be copied,
+# modified or distributed except as expressly authorized under the terms
+# of the license contained in the file LICENSE in this distribution.
+# 
+# For more information about licensing, please refer to
+# http://www.ghostscript.com/licensing/. For information on
+# commercial licensing, go to http://www.artifex.com/licensing/ or
+# contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+# San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 
-
-set TMAKE_ID {$Id: tmake.tcl,v 1.5 2007/08/01 14:26:56 jemarch Exp $}
+set TMAKE_ID {$Id$}
 
 # tmake is intended to be a drop-in replacement for a large and
 # useful subset of 'make'.  It compiles makefiles into Tcl scripts
